@@ -22,8 +22,7 @@ from AnyQt.QtWidgets import QStyledItemDelegate, QStyleOptionViewItem, \
     QApplication, QStyle
 
 from orangewidget.utils.cache import LRUCache
-from orangewidget.utils import enum_as_int
-
+from orangewidget.utils import enum_as_int, grapheme_slice
 
 A = TypeVar("A")
 
@@ -399,6 +398,7 @@ class DataDelegate(CachedDataItemDelegate, StyledItemDelegate):
         self.__static_text_lru_cache = LRUCache(100 * 200)
         self.__pen_lru_cache: LRUCache[_PenKey, QPen] = LRUCache(100)
         self.__style = None
+        self.__max_text_length = 500
 
     def initStyleOption(
             self, option: QStyleOptionViewItem, index: QModelIndex
@@ -423,11 +423,11 @@ class DataDelegate(CachedDataItemDelegate, StyledItemDelegate):
         # Keep ref to style wrapper. This is ugly, wrong but the wrapping of
         # C++ QStyle instance takes ~5% unless the wrapper already exists.
         self.__style = style
-        text = opt.text
-        opt.text = ""
-        style.drawControl(QStyle.CE_ItemViewItem, opt, painter, widget)
-        trect = style.subElementRect(QStyle.SE_ItemViewItemText, opt, widget)
-        opt.text = text
+        # Draw empty item cell
+        opt_c = QStyleOptionViewItem(opt)
+        opt_c.text = ""
+        style.drawControl(QStyle.CE_ItemViewItem, opt_c, painter, widget)
+        trect = style.subElementRect(QStyle.SE_ItemViewItemText, opt_c, widget)
         self.drawViewItemText(style, painter, opt, trect)
 
     def drawViewItemText(
@@ -441,8 +441,9 @@ class DataDelegate(CachedDataItemDelegate, StyledItemDelegate):
             QStyle.PM_FocusFrameHMargin, None, option.widget) + 1
         rect = rect.adjusted(margin, 0, -margin, 0)
         font = option.font
+        text = option.text
         st = self.__static_text_elided_cache(
-            option.text, font, option.fontMetrics, option.textElideMode,
+            text, font, option.fontMetrics, option.textElideMode,
             rect.width()
         )
         tsize = st.size()
@@ -477,13 +478,23 @@ class DataDelegate(CachedDataItemDelegate, StyledItemDelegate):
         try:
             return self.__static_text_lru_cache[text, font, elideMode, width]
         except KeyError:
-            st = QStaticText(fontMetrics.elidedText(text, elideMode, width))
+            # limit text to some sensible length in case it is a whole epic
+            # tale or similar. elidedText will parse all of it to glyphs which
+            # can be slow.
+            text_limited = self.__cut_text(text)
+            st = QStaticText(fontMetrics.elidedText(text_limited, elideMode, width))
             st.prepare(QTransform(), font)
             # take a copy of the font for cache key
             key = text, QFont(font), elideMode, width
             self.__static_text_lru_cache[key] = st
             return st
 
+    def __cut_text(self, text):
+        if len(text) > self.__max_text_length:
+            return grapheme_slice(text, end=self.__max_text_length)
+        else:
+            return text
+
     def __pen_cache(self, palette: QPalette, state: QStyle.State) -> QPen:
         """Return a QPen from the `palette` for `state`."""
         # NOTE: This method exists mostly to avoid QPen, QColor (de)allocations.
