@@ -2,23 +2,26 @@ import sys
 import enum
 import base64
 from itertools import chain
-from operator import attrgetter
 from xml.sax.saxutils import escape
 from collections import OrderedDict
 from typing import (
     NamedTuple, Tuple, List, Dict, Iterable, Union, Optional, Hashable
 )
 
-from AnyQt.QtCore import Qt, QSize, QBuffer, QPropertyAnimation, QEasingCurve, Property
-from AnyQt.QtGui import (
-    QIcon, QPixmap, QPainter, QPalette, QLinearGradient, QBrush, QPen
+from AnyQt.QtCore import (
+    Qt, QSize, QBuffer, QPropertyAnimation, QEasingCurve, Property
 )
+from AnyQt.QtGui import QIcon, QPixmap, QPainter
 from AnyQt.QtWidgets import (
-    QWidget, QLabel, QSizePolicy, QStyle, QHBoxLayout, QMessageBox,
+    QWidget, QLabel, QSizePolicy, QStyle, QHBoxLayout,
     QMenu, QWidgetAction, QStyleOption, QStylePainter, QApplication
 )
 from AnyQt.QtCore import pyqtSignal as Signal
 
+from orangecanvas.utils.localization import pl
+
+from orangewidget.utils.buttons import flat_button_hover_background
+
 __all__ = ["Message", "MessagesWidget"]
 
 
@@ -50,11 +53,11 @@ class Severity(enum.IntEnum):
     An enum defining a severity level.
     """
     #: General informative message.
-    Information = QMessageBox.Information
+    Information = 1  # == QMessageBox.Information
     #: A warning message severity.
-    Warning = QMessageBox.Warning
+    Warning = 2  # == QMessageBox.Warning
     #: An error message severity.
-    Error = QMessageBox.Critical
+    Error = 3  # == QMessageBox.Critical
 
 
 class Message(
@@ -103,6 +106,9 @@ class Message(
         return super().__new__(cls, Severity(severity), QIcon(icon), text,
                                informativeText, detailedText, textFormat)
 
+    def __bool__(self):
+        return not self.isEmpty()
+
     def asHtml(self, includeShortText=True):
         # type: () -> str
         """
@@ -152,6 +158,7 @@ class Message(
         """
         return (not self.text and self.icon.isNull() and
                 not self.informativeText and not self.detailedText)
+
     @property
     def icon(self):
         return QIcon(super().icon)
@@ -268,19 +275,16 @@ def summarize(messages):
     elif warnings:
         severity = Severity.Warning
 
-    def format_plural(fstr, items, *args, **kwargs):
-        return fstr.format(len(items), *args,
-                           s="s" if len(items) != 1 else "",
-                           **kwargs)
+    nerrors, nwarnings, ninfo = len(errors), len(warnings), len(info)
     if errors:
-        text_parts.append(format_plural("{} error{s}", errors))
+        text_parts.append(f"{nerrors} {pl(nerrors, 'error')}")
     if warnings:
-        text_parts.append(format_plural("{} warning{s}", warnings))
+        text_parts.append(f"{nwarnings} {pl(nwarnings, 'warning')}")
     if info:
         if not (errors and warnings and lead):
-            text_parts.append(format_plural("{} message{s}", info))
+            text_parts.append(f"{ninfo} {pl(ninfo, 'message')}")
         else:
-            text_parts.append(format_plural("{} other", info))
+            text_parts.append(f"{ninfo} other {pl(ninfo, 'message')}")
 
     if leading_text:
         text = leading_text
@@ -338,13 +342,11 @@ class ElidingLabel(QLabel):
             super().setText(self.__originalText)
 
 
-class MessagesWidget(QWidget):
+class MessageWidget(QWidget):
     """
-    An iconified multiple message display area.
+    An iconified message display area.
 
-    `MessagesWidget` displays a short message along with an icon. If there
-    are multiple messages they are summarized. The user can click on the
-    widget to display the full message text in a popup view.
+    `IconifiedMessage` displays a short message along with an icon.
     """
     #: Signal emitted when an embedded html link is clicked
     #: (if `openExternalLinks` is `False`).
@@ -353,14 +355,6 @@ class MessagesWidget(QWidget):
     #: Signal emitted when an embedded html link is hovered.
     linkHovered = Signal(str)
 
-    Severity = Severity
-    #: General informative message.
-    Information = Severity.Information
-    #: A warning message severity.
-    Warning = Severity.Warning
-    #: An error message severity.
-    Error = Severity.Error
-
     Message = Message
 
     def __init__(self, parent=None, openExternalLinks=False, elideText=False,
@@ -370,11 +364,10 @@ class MessagesWidget(QWidget):
             QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Minimum)
         )
         super().__init__(parent, **kwargs)
-        self.__openExternalLinks = openExternalLinks  # type: bool
-        self.__messages = OrderedDict()  # type: Dict[Hashable, Message]
+        self._openExternalLinks = openExternalLinks  # type: bool
         #: The full (joined all messages text - rendered as html), displayed
         #: in a tooltip.
-        self.__fulltext = ""
+        self.message = None
         #: Leading icon
         self.__iconwidget = IconWidget(
             sizePolicy=QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
@@ -383,7 +376,7 @@ class MessagesWidget(QWidget):
         self.__textlabel = ElidingLabel(
             wordWrap=False,
             textInteractionFlags=Qt.LinksAccessibleByMouse,
-            openExternalLinks=self.__openExternalLinks,
+            openExternalLinks=self._openExternalLinks,
             sizePolicy=QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Minimum),
             elide=elideText
         )
@@ -407,17 +400,30 @@ class MessagesWidget(QWidget):
         self.anim.setEasingCurve(QEasingCurve.OutQuad)
         self.anim.setLoopCount(2)
 
+    def setMessage(self, message):
+        self.message = message
+        self.ensurePolished()
+        icon = message_icon(message)
+        self.__iconwidget.setIcon(icon)
+        self.__iconwidget.setVisible(not (message.isEmpty() or icon.isNull()))
+        self.__textlabel.setTextFormat(message.textFormat)
+        self.__textlabel.setText(message.text)
+        self.__textlabel.setVisible(bool(message.text))
+        self.setToolTip(self._styled(message.asHtml()))
+        self.anim.start(QPropertyAnimation.KeepWhenStopped)
+        self.layout().activate()
+
     def sizeHint(self):
         sh = super().sizeHint()
         h = self.style().pixelMetric(QStyle.PM_SmallIconSize)
-        if all(m.isEmpty() for m in self.messages()):
+        if not self.message:
             sh.setWidth(0)
         return sh.expandedTo(QSize(0, h + 2))
 
     def minimumSizeHint(self):
         msh = super().minimumSizeHint()
         h = self.style().pixelMetric(QStyle.PM_SmallIconSize)
-        if all(m.isEmpty() for m in self.messages()):
+        if not self.message:
             msh.setWidth(0)
         else:
             msh.setWidth(h + 2)
@@ -431,14 +437,14 @@ class MessagesWidget(QWidget):
         using `QDesktopServices.openUrl`
         """
         # TODO: update popup if open
-        self.__openExternalLinks = state
+        self._openExternalLinks = state
         self.__textlabel.setOpenExternalLinks(state)
 
     def openExternalLinks(self):
         # type: () -> bool
         """
         """
-        return self.__openExternalLinks
+        return self._openExternalLinks
 
     def setDefaultStyleSheet(self, css):
         # type: (str) -> None
@@ -463,7 +469,6 @@ class MessagesWidget(QWidget):
         """
         if self.__defaultStyleSheet != css:
             self.__defaultStyleSheet = css
-            self.__update()
 
     def defaultStyleSheet(self):
         """
@@ -474,6 +479,61 @@ class MessagesWidget(QWidget):
         """
         return self.__defaultStyleSheet
 
+    def flashIcon(self):
+        self.anim.start(QPropertyAnimation.KeepWhenStopped)
+
+    def _styled(self, html):
+        # Prepend css style sheet before a html fragment.
+        if self.__defaultStyleSheet.strip():
+            return f"<style>{escape(self.__defaultStyleSheet)}</style>\n{html}"
+        else:
+            return html
+
+    def enterEvent(self, event):
+        super().enterEvent(event)
+        self.update()
+
+    def leaveEvent(self, event):
+        super().leaveEvent(event)
+        self.update()
+
+    def changeEvent(self, event):
+        super().changeEvent(event)
+        self.update()
+
+    def paintEvent(self, event):
+        if not self.message:
+            return
+
+        opt = QStyleOption()
+        opt.initFrom(self)
+        if opt.state & (QStyle.State_MouseOver | QStyle.State_HasFocus):
+            p = QPainter(self)
+            flat_button_hover_background(p, opt)
+
+
+class MessagesWidget(MessageWidget):
+    """
+    An iconified multiple message display area.
+
+    `MessagesWidget` displays a short message along with an icon. If there
+    are multiple messages they are summarized. The user can click on the
+    widget to display the full message text in a popup view.
+    """
+    Severity = Severity
+    #: General informative message.
+    Information = Severity.Information
+    #: A warning message severity.
+    Warning = Severity.Warning
+    #: An error message severity.
+    Error = Severity.Error
+
+    def __init__(self, parent=None, openExternalLinks=False, elideText=False,
+                 defaultStyleSheet="", **kwargs):
+        super().__init__(parent, openExternalLinks, elideText,
+                         defaultStyleSheet, **kwargs)
+        self.__messages = OrderedDict()  # type: Dict[Hashable, Message]
+
     def setMessage(self, message_id, message):
         # type: (Hashable, Message) -> None
         """
@@ -547,54 +607,20 @@ class MessagesWidget(QWidget):
                 self.anim.start(QPropertyAnimation.KeepWhenStopped)
                 break
 
-    @staticmethod
-    def __styled(css, html):
-        # Prepend css style sheet before a html fragment.
-        if css.strip():
-            return "<style>\n" + escape(css) + "\n</style>\n" + html
-        else:
-            return html
-
     def __update(self):
-        """
-        Update the current display state.
-        """
-        self.ensurePolished()
-        summary = self.summarize()
-        icon = message_icon(summary)
-        self.__iconwidget.setIcon(icon)
-        self.__iconwidget.setVisible(not (summary.isEmpty() or icon.isNull()))
-        self.__textlabel.setTextFormat(summary.textFormat)
-        self.__textlabel.setText(summary.text)
-        self.__textlabel.setVisible(bool(summary.text))
-
-        def is_short(m):
-            return not (m.informativeText or m.detailedText)
-
-        messages = [m for m in self.__messages.values() if not m.isEmpty()]
-        if not messages:
-            fulltext = ""
-        else:
-            messages = sorted(messages, key=attrgetter("severity"),
-                              reverse=True)
-            fulltext = "<hr/>".join(m.asHtml() for m in messages)
-        self.__fulltext = fulltext
-        self.setToolTip(self.__styled(self.__defaultStyleSheet, fulltext))
-        self.anim.start(QPropertyAnimation.KeepWhenStopped)
-
-        self.layout().activate()
+        super().setMessage(self.summarize())
 
     def mousePressEvent(self, event):
         if event.button() == Qt.LeftButton:
-            if self.__fulltext:
+            message = self.message
+            if message:
                 popup = QMenu(self)
                 label = QLabel(
                     self, textInteractionFlags=Qt.TextBrowserInteraction,
-                    openExternalLinks=self.__openExternalLinks,
+                    openExternalLinks=self._openExternalLinks,
                 )
                 label.setContentsMargins(4, 4, 4, 4)
-                label.setText(self.__styled(self.__defaultStyleSheet,
-                                            self.__fulltext))
+                label.setText(self._styled(message.asHtml()))
 
                 label.linkActivated.connect(self.linkActivated)
                 label.linkHovered.connect(self.linkHovered)
@@ -607,51 +633,17 @@ class MessagesWidget(QWidget):
         else:
             super().mousePressEvent(event)
 
-    def enterEvent(self, event):
-        super().enterEvent(event)
-        self.update()
 
-    def leaveEvent(self, event):
-        super().leaveEvent(event)
-        self.update()
+class InOutStateWidget(MessageWidget):
+    clicked = Signal()
 
-    def changeEvent(self, event):
-        super().changeEvent(event)
-        self.update()
-
-    def paintEvent(self, event):
-        opt = QStyleOption()
-        opt.initFrom(self)
-        if not self.__fulltext:
+    def mousePressEvent(self, event):
+        if event.button() == Qt.LeftButton:
+            self.clicked.emit()
+            event.accept()
             return
-
-        if not (opt.state & QStyle.State_MouseOver or
-                opt.state & QStyle.State_HasFocus):
-            return
-
-        palette = opt.palette  # type: QPalette
-        if opt.state & QStyle.State_HasFocus:
-            pen = QPen(palette.color(QPalette.Highlight))
         else:
-            pen = QPen(palette.color(QPalette.Dark))
-
-        if self.__fulltext and \
-                opt.state & QStyle.State_MouseOver and \
-                opt.state & QStyle.State_Active:
-            g = QLinearGradient()
-            g.setCoordinateMode(QLinearGradient.ObjectBoundingMode)
-            base = palette.color(QPalette.Window)
-            base.setAlpha(90)
-            g.setColorAt(0, base.lighter(200))
-            g.setColorAt(0.6, base)
-            g.setColorAt(1.0, base.lighter(200))
-            brush = QBrush(g)
-        else:
-            brush = QBrush(Qt.NoBrush)
-        p = QPainter(self)
-        p.setBrush(brush)
-        p.setPen(pen)
-        p.drawRect(opt.rect.adjusted(0, 0, -1, -1))
+            super().mousePressEvent(event)
 
 
 class IconWidget(QWidget):
@@ -716,8 +708,10 @@ class IconWidget(QWidget):
         painter.drawPrimitive(QStyle.PE_Widget, opt)
         if not self.__icon.isNull():
             rect = self.contentsRect()
-            if opt.state & QStyle.State_Active:
+            if opt.state & QStyle.State_MouseOver:
                 mode = QIcon.Active
+            elif opt.state & QStyle.State_Enabled:
+                mode = QIcon.Normal
             else:
                 mode = QIcon.Disabled
             self.__icon.paint(painter, rect, Qt.AlignCenter, mode, QIcon.Off)
@@ -776,5 +770,6 @@ def main(argv=None):  # pragma: no cover
     w.show()
     return app.exec()
 
+
 if __name__ == "__main__":  # pragma: no cover
     sys.exit(main(sys.argv))
